package com.caimei.controller;

import com.caimei.model.ResponseJson;
import com.caimei.model.vo.HeHeActivityVo;
import com.caimei.model.vo.ProductVo;
import com.caimei.service.ProductService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/22
 */
@Api(tags = "商品管理API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/product")
public class ProductApi {
    private final ProductService productService;

    @ApiOperation("轮播图")
    @GetMapping("/carousel")
    public ResponseJson<List<String>> carouselFigure() {
        return productService.carouselFigure();
    }

    @ApiOperation("商品列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name", value = "搜索商品名称", required = false),
            @ApiImplicitParam(name = "userId", value = "用户id", required = false),
            @ApiImplicitParam(name = "pageNum", value = "第几页", required = false),
            @ApiImplicitParam(name = "pageSize", value = "一页多少条", required = false)
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ProductVo>> productList(String name, Integer userId, @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                         @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        return productService.productList(name, userId, pageNum, pageSize);
    }

    @ApiOperation("商品详情")
    @ApiImplicitParam(name = "productId", required = true, value = "商品Id")
    @GetMapping("/details")
    public ResponseJson<ProductVo> productDetails(Integer productId) {
        return productService.productDetails(productId);
    }

    @ApiOperation("商品搜索历史记录")
    @ApiImplicitParam(name = "userId", required = true, value = "用户id")
    @GetMapping("/search/history")
    public ResponseJson<List<String>> searchHistory(Integer userId) {
        if (userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return productService.searchHistory(userId);
    }

    @ApiOperation("删除搜索历史记录")
    @ApiImplicitParam(name = "userId", required = true, value = "用户id")
    @GetMapping("/delete/history")
    public ResponseJson<String> deleteHistory(Integer userId) {
        if (userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return productService.deleteHistory(userId);
    }

    @ApiOperation("活动专区")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "分销者用户id", required = true),
            @ApiImplicitParam(name = "pageNum", value = "第几页", required = false),
            @ApiImplicitParam(name = "pageSize", value = "一页多少条", required = false)
    })
    @GetMapping("/activityArea")
    public ResponseJson<PageInfo<HeHeActivityVo>> activityArea(Integer userId, @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                               @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        if (userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return productService.activityArea(userId, pageNum, pageSize);
    }

    @ApiOperation("活动详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "分销者用户id", required = true),
            @ApiImplicitParam(name = "activityId", value = "活动id", required = true),
            @ApiImplicitParam(name = "pageNum", value = "第几页", required = false),
            @ApiImplicitParam(name = "pageSize", value = "一页多少条", required = false)
    })
    @GetMapping("/activity/details")
    public ResponseJson<Map<String, Object>> activityDetails(Integer userId, Integer activityId,
                                                             @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                             @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        if (userId == null || activityId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return productService.activityDetails(userId, activityId, pageNum, pageSize);
    }
}
