package com.caimei.mapper;

import com.caimei.model.po.CmDiscernReceiptPo;
import com.caimei.model.po.CmPayShopPo;
import com.caimei.model.po.CmPayShopRecordPo;
import com.caimei.model.po.CmReceiptOrderRelationPo;
import com.caimei.model.vo.DiscernReceiptVo;
import com.caimei.model.vo.OrderProductVo;
import com.caimei.model.vo.OrderVo;
import com.caimei.model.vo.SplitAccountVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/26
 */
@Mapper
public interface PayOrderMapper {
    /**
     * 获取线上支付开关状态
     *
     * @return
     */
    Integer getPayOnLineSwitch();

    /**
     * 查询支付记录
     *
     * @param order
     * @return
     */
    List<DiscernReceiptVo> getDiscernReceipt(OrderVo order);

    /**
     * 更新订单信息
     *
     * @param order
     */
    void updateSelective(OrderVo order);

    /**
     * 保存收款记录
     *
     * @param discernReceipt
     */
    void insertDiscernReceipt(CmDiscernReceiptPo discernReceipt);

    /**
     * 保存订单收款关系
     *
     * @param relation
     */
    void insertOrderRelation(CmReceiptOrderRelationPo relation);

    /**
     * 查询订单商品
     *
     * @param orderId
     * @return
     */
    List<OrderProductVo> findAllOrderProduct(Integer orderId);

    /**
     * 查询未分账已支付收款
     *
     * @param currentTime 延迟时间
     * @return
     */
    List<CmReceiptOrderRelationPo> findUnallocatedAccount(String currentTime);

    /**
     * 保存分账信息
     *
     * @param splitAccount
     */
    void insertSplitAccount(SplitAccountVo splitAccount);

    /**
     * 查询已支付金额
     *
     * @param orderProductId
     * @return
     */
    BigDecimal findPaidAmount(Integer orderProductId);

    /**
     * 查询子商户商编
     *
     * @param shopId
     * @return
     */
    String findCommercialCode(Long shopId);

    /**
     * 查询已支付运费
     *
     * @param orderId
     * @param shopId
     * @return
     */
    BigDecimal findShipping(@Param("orderId") Integer orderId, @Param("shopId") Integer shopId);

    /**
     * 修改收款分账状态
     *
     * @param mbOrderId
     */
    void updateBySplitStatus(String mbOrderId);

    /**
     * 查询分账信息
     *
     * @param mbOrderId 米花平台交易流水号
     * @return
     */
    List<SplitAccountVo> findByMbOrderId(String mbOrderId);

    /**
     * 查询已付供应商金额
     *
     * @param shopOrderId
     * @return
     */
    BigDecimal findPaidShop(Integer shopOrderId);

    /**
     * 修改子订单付款状态及付款金额
     *
     * @param shopOrderId
     * @param paidShop
     * @param payStatus
     */
    void updateShopOrderByPayStatus(@Param("shopOrderId") Integer shopOrderId, @Param("paidShop") BigDecimal paidShop, @Param("payStatus") String payStatus);

    /**
     * 保存付款单
     *
     * @param payShop
     */
    void insertPayShop(CmPayShopPo payShop);

    /**
     * 保存付款记录
     *
     * @param shopRecord
     */
    void insertPayShopRecord(CmPayShopRecordPo shopRecord);

    /**
     * 修改主订单付款状态
     *
     * @param orderId
     * @param payStatus
     */
    void updateOrderByPayStatus(@Param("orderId") Integer orderId, @Param("payStatus") String payStatus);
}
