package com.caimei.service;

import com.caimei.model.ResponseJson;
import com.caimei.model.vo.LogisticsBatchVo;
import com.caimei.model.vo.OrderProductVo;
import com.caimei.model.vo.OrderVo;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/26
 */
public interface OrderService {
    /**
     * 订单数据
     *
     * @param orderState
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    ResponseJson<PageInfo<OrderVo>> getOrderList(Integer orderState, Integer userId, Integer pageNum, Integer pageSize);

    /**
     * 订单详情
     *
     * @param orderId
     * @return
     */
    ResponseJson<Map<String, Object>> getOrderDetail(Integer orderId);

    /**
     * 物流信息
     *
     * @param orderId
     * @return
     */
    ResponseJson<List<LogisticsBatchVo>> logistics(Integer orderId);

    /**
     * 确认收货
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> affirmCargo(Integer orderId);

    /**
     * 删除订单
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> deleteOrder(Integer orderId);

    /**
     * 取消订单
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> cancelOrder(Integer orderId);

    /**
     * 根据关键词获取订单
     *
     * @param searchWord
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    ResponseJson<PageInfo<OrderVo>> searchOrder(String searchWord, Integer userId, Integer pageNum, Integer pageSize);

    /**
     * 根据组织ID查找历史记录
     *
     * @param userId
     * @return
     */
    ResponseJson<List<String>> getSearchHistory(Integer userId);

    /**
     * 删除历史记录
     *
     * @param userId
     * @return
     */
    ResponseJson<String> deleteSearchHistory(Integer userId);

    /**
     * 订单数量统计
     *
     * @param userId
     * @return
     */
    ResponseJson<Map<String, Object>> orderTotal(Integer userId);

    /**
     * 分销者订单列表
     *
     * @param orderState
     * @param orderType
     * @param userId
     * @param searchWord
     * @param pageNum
     * @param pageSize
     * @return
     */
    ResponseJson<PageInfo<OrderVo>> dealerOrderList(Integer orderState, Integer orderType, Integer userId, String searchWord, Integer pageNum, Integer pageSize);

    /**
     * 再次购买
     *
     * @param orderId
     * @return
     */
    ResponseJson<List<OrderProductVo>> buyAgain(Integer orderId);
}
